@@ -111,14 +111,27 @@ void store_layer (GtkLayerShellLayer layer)
     orig_layer = layer;
 }
 
+void set_image_from_pixbuf (GtkWidget *image, GdkPixbuf *pixbuf)
+{
+    int scale = gtk_widget_get_scale_factor (image);
+    if (scale == 1) gtk_image_set_from_pixbuf (GTK_IMAGE (image), pixbuf);
+    else
+    {
+        cairo_surface_t *cr = gdk_cairo_surface_create_from_pixbuf (pixbuf, scale, NULL);
+        gtk_image_set_from_surface (GTK_IMAGE (image), cr);
+        cairo_surface_destroy (cr);
+    }
+}
+
 void set_taskbar_icon (GtkWidget *image, const char *icon, int size)
 {
     if (!icon) return;
-    GdkPixbuf *pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (), icon,
-        size, GTK_ICON_LOOKUP_FORCE_SIZE, NULL);
+    int scale = gtk_widget_get_scale_factor (image);
+    GdkPixbuf *pixbuf = gtk_icon_theme_load_icon_for_scale (gtk_icon_theme_get_default (), icon,
+        size, scale, GTK_ICON_LOOKUP_FORCE_SIZE, NULL);
     if (pixbuf)
     {
-        gtk_image_set_from_pixbuf (GTK_IMAGE (image), pixbuf);
+        set_image_from_pixbuf (image, pixbuf);
         g_object_unref (pixbuf);
     }
 }
@@ -126,11 +139,12 @@ void set_taskbar_icon (GtkWidget *image, const char *icon, int size)
 void set_menu_icon (GtkWidget *image, const char *icon, int size)
 {
     if (!icon) return;
-    GdkPixbuf *pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (), icon,
-        size >= 32 ? 24 : 16, GTK_ICON_LOOKUP_FORCE_SIZE, NULL);
+    int scale = gtk_widget_get_scale_factor (image);
+    GdkPixbuf *pixbuf = gtk_icon_theme_load_icon_for_scale (gtk_icon_theme_get_default (), icon,
+        (size >= 32 ? 24 : 16), scale, GTK_ICON_LOOKUP_FORCE_SIZE, NULL);
     if (pixbuf)
     {
-        gtk_image_set_from_pixbuf (GTK_IMAGE (image), pixbuf);
+        set_image_from_pixbuf (image, pixbuf);
         g_object_unref (pixbuf);
     }
 }
@@ -205,8 +219,10 @@ static void graph_redraw (PluginGraph *graph, char *label)
     unsigned int fontsize, drawing_cursor, i;
     GdkPixbuf *pixbuf;
 
+    uint scale = gtk_widget_get_scale_factor (graph->da);
+
     cairo_t *cr = cairo_create (graph->pixmap);
-    cairo_set_line_width (cr, 1.0);
+    cairo_set_line_width (cr, scale);
     
     /* Erase pixmap */
     cairo_rectangle (cr, 0, 0, graph->pixmap_width, graph->pixmap_height);
@@ -215,7 +231,7 @@ static void graph_redraw (PluginGraph *graph, char *label)
 
     /* Recompute pixmap */
     drawing_cursor = graph->ring_cursor;
-    for (i = 0; i < graph->pixmap_width; i++)
+    for (i = 0; i < graph->pixmap_width / scale; i++)
     {
         /* Draw one bar of the graph. */
         if (graph->samples[drawing_cursor] != 0.0)
@@ -223,33 +239,35 @@ static void graph_redraw (PluginGraph *graph, char *label)
             cairo_set_source_rgba (cr, graph->colours[graph->samp_states[drawing_cursor]].blue, graph->colours[graph->samp_states[drawing_cursor]].green,
                 graph->colours[graph->samp_states[drawing_cursor]].red, graph->colours[graph->samp_states[drawing_cursor]].alpha);
 
-            cairo_move_to (cr, i + 0.5, graph->pixmap_height);
-            cairo_line_to (cr, i + 0.5, graph->pixmap_height - graph->samples[drawing_cursor] * graph->pixmap_height);
+            cairo_move_to (cr, i * scale + 0.5, graph->pixmap_height - scale);
+            cairo_line_to (cr, i * scale + 0.5, graph->pixmap_height - scale - graph->samples[drawing_cursor] * (graph->pixmap_height - 2 * scale));
             cairo_stroke (cr);
         }
 
         /* Increment and wrap drawing cursor */
         drawing_cursor += 1;
-        if (drawing_cursor >= graph->pixmap_width) drawing_cursor = 0;
+        if (drawing_cursor >= graph->pixmap_width / scale) drawing_cursor = 0;
     }
 
     /* Draw border in black */
     cairo_set_source_rgb (cr, 0, 0, 0);
-    cairo_set_line_width (cr, 1);
-    cairo_move_to (cr, 0, 0);
-    cairo_line_to (cr, 0, graph->pixmap_height);
-    cairo_line_to (cr, graph->pixmap_width, graph->pixmap_height);
-    cairo_line_to (cr, graph->pixmap_width, 0);
-    cairo_line_to (cr, 0, 0);
+    cairo_set_line_width (cr, scale);
+    cairo_move_to (cr, scale - 1, scale - 1);
+    cairo_line_to (cr, scale - 1, graph->pixmap_height - scale + 1);
+    cairo_line_to (cr, graph->pixmap_width - scale + 1, graph->pixmap_height - scale + 1);
+    cairo_line_to (cr, graph->pixmap_width - scale + 1, scale - 1);
+    cairo_line_to (cr, scale - 1, scale - 1);
     cairo_stroke (cr);
 
     /* Apply label */
-    fontsize = 12;
-    if (graph->pixmap_width > 50) fontsize = graph->pixmap_height / 3;
+    fontsize = 12 * scale;
+    if (graph->pixmap_width > 50 * scale) fontsize = graph->pixmap_height / 3;
     cairo_select_font_face (cr, "monospace", CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_NORMAL);
     cairo_set_font_size (cr, fontsize);
     cairo_set_source_rgb (cr, 0, 0, 0);
-    cairo_move_to (cr, (graph->pixmap_width >> 1) - ((fontsize * 5) / 4), ((graph->pixmap_height + fontsize) >> 1) - 1);
+    cairo_text_extents_t extents;
+    cairo_text_extents (cr, label, &extents);
+    cairo_move_to (cr, (graph->pixmap_width - extents.width) / 2, (graph->pixmap_height + extents.height) / 2);
     cairo_show_text (cr, label);
 
     cairo_destroy (cr);
@@ -257,7 +275,14 @@ static void graph_redraw (PluginGraph *graph, char *label)
     /* Update image */
     pixbuf = gdk_pixbuf_new_from_data (cairo_image_surface_get_data (graph->pixmap), GDK_COLORSPACE_RGB, TRUE, 8, 
         graph->pixmap_width, graph->pixmap_height, graph->pixmap_width * 4, NULL, NULL);
-    gtk_image_set_from_pixbuf (GTK_IMAGE (graph->da), pixbuf);
+
+    if (scale == 1) gtk_image_set_from_pixbuf (GTK_IMAGE (graph->da), pixbuf);
+    else
+    {
+        cairo_surface_t *cr = gdk_cairo_surface_create_from_pixbuf (pixbuf, scale, NULL);
+        gtk_image_set_from_surface (GTK_IMAGE (graph->da), cr);
+        cairo_surface_destroy (cr);
+    }
     g_object_unref (pixbuf);
 }
 
@@ -265,6 +290,8 @@ static void graph_redraw (PluginGraph *graph, char *label)
 
 void graph_reload (PluginGraph *graph, int icon_size, GdkRGBA background, GdkRGBA foreground, GdkRGBA throttle1, GdkRGBA throttle2)
 {
+    int scale = gtk_widget_get_scale_factor (graph->da);
+
     /* Load colours */
     graph->background = background;
     graph->colours[0] = foreground;
@@ -276,6 +303,9 @@ void graph_reload (PluginGraph *graph, int icon_size, GdkRGBA background, GdkRGB
     guint new_pixmap_width = (new_pixmap_height * 3) >> 1;
     if (new_pixmap_width < 50) new_pixmap_width = 50;
 
+    new_pixmap_width *= scale;
+    new_pixmap_height *= scale;
+
     if ((new_pixmap_width > 0) && (new_pixmap_height > 0))
     {
         /* If statistics buffer does not exist or it changed size, reallocate and preserve existing data. */
@@ -370,7 +400,7 @@ static void menu_hidden (GtkWidget *, kb_menu_t *data)
 {
     g_signal_handler_disconnect (data->menu, data->mhandle);
     gtk_layer_set_layer (panel, orig_layer);
-    gtk_layer_set_keyboard_interactivity (panel, FALSE);
+    gtk_layer_set_keyboard_mode (panel, GTK_LAYER_SHELL_KEYBOARD_MODE_ON_DEMAND);
     if (data->button) g_idle_add ((GSourceFunc) hide_prelight, data->button);
     g_free (data);
 }
@@ -438,14 +468,14 @@ void show_menu_with_kbd (GtkWidget *widget, GtkWidget *menu)
 
     panel = find_panel (widget);
 
-    if (GTK_IS_BUTTON (widget) || GTK_IS_EVENT_BOX (widget) || GTK_IS_HBOX (widget)) data->button = widget;
+    if (GTK_IS_BUTTON (widget) || GTK_IS_EVENT_BOX (widget) || GTK_IS_BOX (widget)) data->button = widget;
     else data->button = NULL;
     data->menu = GTK_MENU (menu);
     data->x = -1.0;
     data->y = -1.0;
 
     gtk_layer_set_layer (panel, GTK_LAYER_SHELL_LAYER_TOP);
-    gtk_layer_set_keyboard_interactivity (panel, TRUE);
+    gtk_layer_set_keyboard_mode (panel, GTK_LAYER_SHELL_KEYBOARD_MODE_EXCLUSIVE);
     data->chandle = g_signal_connect (gtk_widget_get_window (GTK_WIDGET (panel)), "committed", G_CALLBACK (committed), data);
 }
 
@@ -463,7 +493,7 @@ void show_menu_with_kbd_at_xy (GtkWidget *widget, GtkWidget *menu, double x, dou
     data->y = y;
 
     gtk_layer_set_layer (panel, GTK_LAYER_SHELL_LAYER_TOP);
-    gtk_layer_set_keyboard_interactivity (panel, TRUE);
+    gtk_layer_set_keyboard_mode (panel, GTK_LAYER_SHELL_KEYBOARD_MODE_EXCLUSIVE);
     data->chandle = g_signal_connect (gtk_widget_get_window (GTK_WIDGET (panel)), "committed", G_CALLBACK (committed), data);
 }
 
@@ -656,7 +686,7 @@ void popup_window_at_button (GtkWidget *window, GtkWidget *button)
         gtk_layer_set_margin (popwindow, GTK_LAYER_SHELL_EDGE_LEFT, px);
     }
     gtk_layer_set_monitor (popwindow, mon);
-    gtk_layer_set_keyboard_interactivity (popwindow, TRUE);
+    gtk_layer_set_keyboard_mode (popwindow, GTK_LAYER_SHELL_KEYBOARD_MODE_EXCLUSIVE);
 
     gtk_window_present (popwindow);
 
