@@ -5,9 +5,9 @@ extern "C" {
     void destroy (WayfireWidget *w) { delete w; }
 
     static constexpr conf_table_t conf_table[3] = {
-        {CONF_INT,  "smooth_scrolling_threshold",  N_("Smooth scrolling threshold")},
-        {CONF_BOOL, "menu_on_middle_click",        N_("Middle button activates menu")},
-        {CONF_NONE, NULL,               NULL}
+        {CONF_TYPE_INT,     "smooth_scrolling_threshold",   N_("Smooth scrolling threshold"),   NULL},
+        {CONF_TYPE_BOOL,    "menu_on_middle_click",         N_("Middle button activates menu"), NULL},
+        {CONF_TYPE_NONE,    NULL,                           NULL,                               NULL}
     };
     const conf_table_t *config_params (void) { return conf_table; };
     const char *display_name (void) { return N_("System Tray"); };
